
presets = {
    'ssh': {
        'command': 'ssh -t',
        'args': "'cd {workdir}; bash -l'",
        'workdir': '$HOME',
    },
    'mysql': {
        'command': 'mysql -u{user} -p{password} -P{port} -h',
        'args': '-A',
        'user': '$USER',
        'password': '',
        'port': '3306',
    },
}




from __future__ import absolute_import
import subprocess
import signal

from . import exceptions

class Host:

    def __init__(self, config):
        self.config = config

    def __repr__(self):
        return self.config['host']

    def __str__(self):
        if 'args' in self.config and self.config.get('alias', False):
            raise exceptions.InvalidConfigException("'args' property is not allowed for alias '%s'" % (repr(self)))

        command = self.config['command']

        if not self.config.get('alias', False):
            command += ' {host}'

        if 'args' in self.config and len(self.config['args']) > 0:
            command += ' ' + self.config['args']

        for variable, value in self.config.items():
            command = command.replace('{' + str(variable) + '}', str(value))

        return command

    def is_match(self, argv, arg_rules, host_rules):
        host = self.config['host']

        for rule_pattern, rule in host_rules.items():
            if rule_pattern in host and not rule(argv):
                return False

        for arg in argv:
            if arg in arg_rules:
                match = arg_rules[arg](host)
            else:
                match = arg in host

            if not match:
                return False

        return True

    def collapse_groups(self, groups):
        group = self.config
        visited = {}
        while group is not None:
            if 'group' in group:
                group_name = group['group']

                if group_name not in groups:
                    raise exceptions.InvalidConfigException("Undefined group '%s'" % (group_name))
                elif group['group'] in visited:
                    raise exceptions.InvalidConfigException("Circular group reference for host '%s'" % (repr(self)))
                else:
                    visited[group_name] = True
                    group = groups[group_name]
                    
                    for key, value in group.items():
                        if key not in self.config:
                            self.config[key] = value
            else:
                group = None

    def run(self):
        child = subprocess.Popen(str(self), shell=True)
        try:
            child.communicate()
        except KeyboardInterrupt:
            child.send_signal(signal.SIGINT)
        return child.returncode




from __future__ import absolute_import
import click
import sys

import hi
from .config import version

@click.command(
    help="""
A configurable shorthand for interfacing with external hosts.\n
\n
Arguments are individually matched against a set of specified hosts. If one result is found the command to interface with that host is run. Otherwise, all matching hosts are printed.
    """,
    epilog="""
\b\bConfigs:\n
\n
Configs are loaded from $HOME/.hi/hosts and $HOME/.hi/groups by default. Configs must be in YAML format.\n
\n
An example host config:\n
\n
  - host: example.com\n
    group: ssh\n
\n
An example group config:\n
  ssh:\n
    command: ssh\n
\n
\b\bRules:\n
\n
Arg rules are applied based on user input. When a user's input matches the name of a specified arg rule, the rule is executed for each host. Arg rules must take a host as an argument and return True or False if the host should be included. A host does not match unless all applicable arg rules return True.\n
\n
Host rules are applied based on the host. When the name of a specified host rule is found in the host, the rule is executed. Host rules must take a tuple of user arguments as an argument and return True or False if the host should be included. A host does not match unless all application host rules return True.
    """,
    context_settings={
        'help_option_names': ['-h','--help'],
    }
)
@click.argument('argv', nargs=-1)
@click.version_option(version=version)
@click.option('--run/--no-run', default=True, help='When enabled, run the command for a host when one match is found.')
@click.option('--hosts-file', help='File to load hosts from. Defaults to $HOME/.hi/hosts.')
@click.option('--groups-file', help='File to load host groups from. Defaults to $HOME/.hi/groups.')
@click.option('--rules/--no-rules', default=True, help='When enabled, use a default set of custom rules to match hosts.')
@click.option('--arg-rule', multiple=True, help='Specify an importable python function to be used as a rule for matching a host. Multiple are allowed. See Rules for more information.')
@click.option('--host-rule', multiple=True, help='Specify an importable python function to be used as a rule for matching a host. Multiple are allowed. See Rules for more information')
def cli(**kwargs):
    kwargs['hosts'] = hi.load_hosts(kwargs['hosts_file'])
    # Don't use the default groups file when using a custom hosts file to avoid unintended behavior/confusion
    if kwargs['hosts_file'] is not None and kwargs['groups_file'] is None:
        kwargs['groups'] = {}
    else:
        kwargs['groups'] = hi.load_groups(kwargs['groups_file'])
    del kwargs['hosts_file']
    del kwargs['groups_file']

    sys.exit(hi.run(**kwargs))

if __name__ == '__main__':
    cli()





class HiException(Exception):

    class_message = 'Failed'

    def __str__(self):
        return '%s: %s' % (self.class_message, super(HiException, self).__str__())

class InvalidConfigException(HiException):

    class_message = 'Invalid config'




import logging
import sys

class InfoFilter(logging.Filter):
  def filter(self, record):
      return record.levelno in (logging.DEBUG, logging.INFO)

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)

handler = logging.StreamHandler(sys.stdout)
handler.setLevel(logging.DEBUG)
handler.addFilter(InfoFilter())
logger.addHandler(handler)

handler = logging.StreamHandler()
handler.setLevel(logging.WARNING)
logger.addHandler(handler)





# Production hosts may not have an explicit stage but will contain a .
prod = lambda host: 'stg' not in host and 'dev' not in host and '.' in host
prd = prod

DEFAULT_ARG_RULES = {
  'prod': prod,
  'prd': prd,
}

# Pad 1-digit searches to avoid matching multiple digit numbers
# 1 should match 01 but not 10
def generate_digit_rule(digit):
    return lambda host: '0' + digit in host
for digit in range(1, 10):
    DEFAULT_ARG_RULES[str(digit)] = generate_digit_rule(str(digit))

# Only match cron or db results if they are explicitly requested
cron = lambda argv: next((arg for arg in argv if 'cron' in arg), None) is not None
db = lambda argv: next((arg for arg in argv if 'db' in arg), None) is not None

DEFAULT_HOST_RULES = {
    'cron': cron,
    'db': db,
}




from setuptools import setup
import importlib

version = importlib.import_module('hi.config').version

setup(
    name='hicli',
    version=version,
    description='A generic CLI for connecting humans to host interfaces.',
    long_description="""
A configurable shorthand for interfacing with external hosts.
Arguments are individually matched against a set of specified hosts. If one result is found the command to interface with that host is run. Otherwise, all matching hosts are printed.
    """,
    author='Matt Wisniewski',
    author_email='hicli@mattw.us',
    license='GPLv3',
    url='https://github.com/polishmatt/hi',
    keywords=['hi', 'cli', 'utility', 'host', 'interface'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
    platforms=['unix','linux'],
    packages=[
        'hi'
    ],
    install_requires=[
        'click==6.7',
        'pyyaml==3.12'
    ],
    entry_points={
        'console_scripts': [
            'hi = hi.cli:cli'
        ],
    },
)



from __future__ import absolute_import
import os
import sys
import importlib

from .rules import DEFAULT_ARG_RULES, DEFAULT_HOST_RULES
from .group import presets
from .host import Host
from .log import logger
from . import exceptions

CONFIG_DIR = os.path.join(os.environ.get('HOME', ''), '.hi')

def load_hosts(file=None):
    hosts = []
    if file is None:
        file = os.path.join(CONFIG_DIR, 'hosts')
    try:
        with open(file) as file:
            import yaml
            hosts = yaml.load(file.read())
    except IOError:
        pass
    return hosts

def load_groups(file=None):
    groups = {}
    if file is None:
        file = os.path.join(CONFIG_DIR, 'groups')
    try:
        with open(file) as file:
            import yaml
            groups = yaml.load(file.read())
    except IOError:
        pass
    return groups

def load_rule(rule):
    last = rule.rfind('.')
    package = rule[:last]
    rule = rule[last + 1:]

    try:
        module = importlib.import_module(package)
    except ImportError:
        if package[:3] == 'hi.':
            package = package[3:]
            module = importlib.import_module(package)

    return (rule, getattr(module, rule))

def load_rules(rules, arg_rule, host_rule):
    if rules:
        arg_rules = DEFAULT_ARG_RULES.copy()
        host_rules = DEFAULT_HOST_RULES.copy()
    else:
        arg_rules = {}
        host_rules = {}
    for rule in arg_rule:
        (rule_pattern, rule_match) = load_rule(rule)
        arg_rules[rule_pattern] = rule_match
    for rule in host_rule:
        (rule_pattern, rule_match) = load_rule(rule)
        host_rules[rule_pattern] = rule_match

    return (arg_rules, host_rules)

def run(argv, hosts, groups, run=True, rules=True, arg_rule=(), host_rule=()):
    argv = list(argv)
    (arg_rules, host_rules) = load_rules(rules, arg_rule, host_rule)

    preset_groups = presets.copy()
    preset_groups.update(groups)
    groups = preset_groups

    try:
        if len(argv) == 0:
            matches = [Host(host_config) for host_config in hosts]
        else:
            matches = []
            for host_config in hosts:
                host = Host(host_config)

                if len(argv) == 1 and repr(host) == argv[0]:
                    matches = [host]
                    break

                if host.is_match(argv, arg_rules, host_rules):
                    matches.append(host)

        if len(matches) == 1:
            match = matches[0]
            match.collapse_groups(groups)
            command = str(match)

            if run:
                return match.run()
            else:
                logger.info(command)
        else:
            for match in matches:
                logger.info(repr(match))

        return 0

    except exceptions.HiException as exception:
        logger.error(str(exception))
        return 1

